import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { AboutPageOurMissionSectionCmsRepository } from "./about-page-our-mission-section-cms.repository";
import { UpdateAboutPageOurMissionSectionCmsDto } from "./dto/update-about-page-our-mission-section-cms.dto";
import { AboutPageOurMissionSectionCmsEntity } from "./entities/about-page-our-mission-section-cms.entity";

@Injectable()
export class AboutPageOurMissionSectionCmsService {
    constructor(
        @InjectRepository(AboutPageOurMissionSectionCmsRepository)
        private readonly repository: AboutPageOurMissionSectionCmsRepository
    ) {}

    async updateOrCreateData(
        dto: UpdateAboutPageOurMissionSectionCmsDto
    ): Promise<AboutPageOurMissionSectionCmsEntity | Error> {
        const isExists = await this.repository.find({ isDeleted: false });
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        } else {
            let id = isExists[0].id;
            const data = await this.repository.updateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<AboutPageOurMissionSectionCmsEntity[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}