import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { UploadedMulterFileI } from "src/modules/SpacesModule/SpacesService";
import { DoSpacesService } from "src/modules/SpacesModule/SpacesService/doSpacesService";
import { TestimonialsRepository } from "./testimonials.repository";
import { CreateTestimonialsDto } from "./dto/create-testimonials.dto";
import { TestimonialsEntity } from "./entities/testimonials.entity";
import { UpdateTestimonialsDto } from "./dto/update-testimonials.dto";

@Injectable()
export class TestimonialsService {
    constructor(
        @InjectRepository(TestimonialsRepository)
        private readonly repository: TestimonialsRepository,
        private readonly doSpaceService: DoSpacesService
    ) {
    }

    async create(
        dto: CreateTestimonialsDto,
        files: {
            image: UploadedMulterFileI
        }
    ): Promise<TestimonialsEntity | Error> {
        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(files.image[0], "Testimonials/");
            dto.image = image;
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
    }

    async update(
        id: string, 
        dto: UpdateTestimonialsDto,
        files: {
            image: UploadedMulterFileI
        }
    ): Promise<TestimonialsEntity | Error> {
        const res: any = await this.repository.findOneEntity(id);
        if (!res) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(files.image[0], "Testimonials/");
            dto.image = image;
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
    }

    async findAll(): Promise<TestimonialsEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<TestimonialsEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully!");
    }
}
